#
# SFR.pm for SMS4NOTHING
#
# Copyright (C) 2001 Cedric Bouvier (cdric) <cbouvi@free.fr>
# Copyright (C) 2000,2001 Julien Gaulmin (julien23) <julien23@multimania.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA or look at http://www.gnu.org/copyleft/gpl.html
#
# Contributors :
#	- Cdric Bouvier (cdric) <cbouvi@free.fr>
#	- Julien Gaulmin (julien23) <julien23@multimania.com>
#
# Change Log :
#	- v0.1 (cdric)   : first version
#	- v0.2 (julien23) : clean-up and options

package Net::SMS::SFR;

use strict;

use HTTP::Cookies;
use Time::localtime;
use Time::Local;
use Date::Manip;
use LWP::UserAgent;
use HTTP::Request;

use constant LANGUAGE => 'FR';
use constant NETWORK => 'smsc1';
use constant VALIDITY_PERIOD => '72';

use vars qw(@ATTR $COOKIE_JAR $UA);

@ATTR = qw(recipient sender message ack date);

sub new {
	my $proto = shift;
	my $class = ref($proto) || $proto;
	my $self = bless {}, $class;

	$self->_set_attr([@ATTR, 'proxy'], @_);

	return $self;
}

sub _set_attr {
	my $self = shift;
	my $permitted_attr_aref = shift;
	my %attr = ();

	if (ref($_[0]) eq 'HASH') {
		%attr = %{$_[0]};
	} else {
		%attr = @_;
	}

	while (my($key, $value) = each %attr) {
		$key =~ s/-//;
		my $pat = qr/^$key/i;

		foreach my $attr (@$permitted_attr_aref) {
			$self->$attr($value) if $attr =~ /$pat/;
		}
	}

	$self->{_sender} ||= $ENV{USER};
}

sub recipient {
	my $self = shift;
	my $old_recipient = $self->{_recipient};

	if (@_) {
		my $new_recipient = shift;

		for ($new_recipient) {
			tr/ \t//d;
			s/^\+33/0/;
			die "Bad phone number format: $_" unless /^06\d{8}$/;

			$self->{_recipient} = $_;
		}
	}

	return $old_recipient;
}

sub message {
	my $self = shift;
	my $old_message = $self->{_message};

	if (@_) {
		my $new_message = shift;
		my $length = length($new_message);

		die "Message is too long: $length char (max 640)"
			if $length > 640;

		$self->{_message} = $new_message;
	}

	return $old_message || '';
}

sub sender {
	my $self = shift;

	return ($self->{_sender} || '', @_ && ($self->{_sender} = shift))[0];
}

sub ack {
	my $self = shift;
	my $old_ack = $self->{_ack};

	if (@_) {
		my $new_ack = shift;
		my $length = length($new_ack);

		die "Bad Email address: $new_ack" unless
			$new_ack =~ /^[\w\.-]+@(?:[\w\.-]+\.)+?[a-z]{2,3}$/i;

		$self->{_ack} = $new_ack;
	}

	return $old_ack || '';
}

sub date {
	my $self = shift;

	return ($self->{_date} || 'now', @_ && ($self->{_date} = shift))[0];
}

sub debug {
	my $self = shift;

	return ($self->{_debug}, @_ && ($self->{_debug} = shift))[0];
}

sub proxy {
	my $self = shift;

	return ($self->{_proxy}, @_ && ($self->{_proxy} = shift))[0];
}

sub verbose {
	my $self = shift;

	return ($self->{_verbose}, @_ && ($self->{_verbose} = shift))[0];
}

sub _init {
	my $self = shift;

	return if $COOKIE_JAR && $UA;

	# Create a cookies jar
	$COOKIE_JAR ||= HTTP::Cookies->new(
		File		=> "$ENV{HOME}/.sms_cookies",
		AutoSave	=> 1
	);

	# Create a user agent object
	unless (defined $UA) {
		$UA = new LWP::UserAgent;

		# Fake Netscape agent on Linux
		$UA->agent('Perlzilla/1.0 (GNU/Linux)');
		$UA->timeout(30);
		if ($self->proxy()) {
			$UA->proxy('http', $self->proxy())
		} else {
			$UA->env_proxy(); # Use proxy defined in ENV by default
		}

		$UA->cookie_jar($COOKIE_JAR);
	}
}

sub send {
	my $self = shift;

	$self->_init();
	$self->_set_attr(\@ATTR, @_);

	#Date_Init("Language=French", "DateFormat=non-US");     # French dates

	my (
		$delivery_year,
		$delivery_month,
		$delivery_date,
		$delivery_hour,
		$delivery_min,
		$delivery_time,
	) = UnixDate((ParseDate($self->date()) || ParseDate('now')),
		qw/%Y %m %d %H %M %s/);

	# SFR compatible
	$delivery_month--;		# january is 0
	$delivery_time .= '000';	# strange sfr epoch format

	print "Year = $delivery_year\nMonth = $delivery_month\n",
		"Date = $delivery_date\nHour = $delivery_hour\n",
		"Min = $delivery_min\nTime = $delivery_time\n"
		if defined $self->{_debug};

	# Create a GET request in order to get the javasession cookie from SFR
	print "Creating the GET request...\n" if defined $self->{_debug};
	my $req = new HTTP::Request;
	$req->method('GET');
	$req->uri('http://195.115.48.10/servlet/ProxyFirst?' .
			'redirect=SMS&LANGUAGE=FR&PAGE=launch');

	# Pass request from the user agent and get a response back
	print "Sending request...\n" if defined $self->{_debug};
	my $res = $UA->request($req);
	$COOKIE_JAR->add_cookie_header($req);

	# Check the outcome of the response
	if ($res->is_success) {
		print "Ok, for the GET part\nContent:\n", $res->content
			if defined $self->{_debug};
	} else {
		select(STDERR);
		print "Error while sending message!\n",
			"May be the remote server is down (timeout).\n",
			"Anyway, check your proxy settings.\n";
		if (defined $self->{_debug}) {
			print "Part:\t\tGET";
			print "\nCode:\t\t", $res->code if defined $res->code;
			print "\nMessage:\t", $res->message
				if defined $res->message;
			print "\nHeader:\t", $res->header
				if defined $res->header;
			print "\nContent:\n", $res->content
				if defined $res->content;
			print "\n";
		}
		exit;
	}

	# Create a POST request to send the message
	print "Creating the POST request...\n" if defined $self->{_debug};
	$req = new HTTP::Request;
	$req->method('POST');
	$req->uri('http://195.115.48.10/servlet/ProxySecond?redirect=SMS');
	$req->referer('http://195.115.48.10/servlet/SMSServlet_KZ1OS9?' .
				'redirect=SMS&LANGUAGE=FR&PAGE=launch');
	$req->content_type('application/x-www-form-urlencoded');

	my %content = (
		NOTIFICATION_FLAG	=> $self->ack() ? 'true' : 'false',
		LANGUAGE		=> LANGUAGE,
		NETWORK			=> NETWORK,
		DELIVERY_TIME		=> $delivery_time,
		DELIVERY_DATE		=> $delivery_date,
		DELIVERY_MONTH		=> $delivery_month,
		DELIVERY_YEAR		=> $delivery_year,
		DELIVERY_HOUR		=> $delivery_hour,
		DELIVERY_MIN		=> $delivery_min,
		SENDER			=> $self->sender(),
		NOTIFICATION_ADDRESS	=> $self->ack(),
		caracteres		=> length($self->message()),
		SHORT_MESSAGE		=> $self->message(),
		VALIDITY_PERIOD		=> VALIDITY_PERIOD,
		RECIPIENT		=> $self->recipient(),
	);
	$req->content(join '&' => map { "$_=$content{$_}" } keys %content);

	# Pass request from the user agent and get a response back
	print "Sending request...\n" if defined $self->{_debug};
	$res = $UA->request($req);
	$COOKIE_JAR->add_cookie_header($req);

	# Check the outcome of the response
	if ($res->is_success) {
		print "Ok, your message has been sent\n"
			if defined $self->{_verbose};
		print "Content:\n", $res->content if defined $self->{_debug};
	} else {
		select(STDERR);
		print "Error while sending message!\n",
			"May be the remote server is down (timeout).\n";
		if (defined $self->{_debug}) {
			print "Part:\t\tPOST";
			print "\nCode:\t\t", $res->code if defined $res->code;
			print "\nMessage:\t", $res->message
				if defined $res->message;
			print "\nHeader:\t", $res->header
				if defined $res->header;
			print "\nContent:\n", $res->content
				if defined $res->content;
			print "\n";
		}
		exit;
	}
}

1;
