#!/usr/bin/perl -w
#
use constant VERSION => 'SMS4NOTHING v0.8';
#
# Copyright (C) 2000,2001 Julien Gaulmin (julien23) <julien23@multimania.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA or look at http://www.gnu.org/copyleft/gpl.html
#
# Contributors :
#	- Julien Gaulmin (julien23) <julien23@multimania.com>
#	- Raphael Rousseau (raph) <raphael.rousseau@fr.alcove.com>
#	- Cdric Bouvier (cdric) <cbouvi@free.fr>
#
# Change Log :
# 	- v0.1 (julien23) : first version
# 	- v0.2 (raph)     : strict implementation
# 	- v0.3 (julien23) : no more Netcape cookies
# 	- v0.4 (julien23) : more flexible command line
# 	- v0.5 (julien23) : get proxy from ENV, use 'if' instead of 'unless !',
#			    better mail address check, timeout for requests
#	- v0.6 (cdric)   : modularization in order to support other operators
#	- v0.7 (julien23) : integration of the new architecture, all options,
#			    clean-up, friendly verbose (-w option)
#	- v0.8 (julien23) : better module searching (-e option), use of
#			    ART file to choose the appropriate operator
#			    (-o and -f options)... Bouygtel support
#			    
#
# To do :
#	- Orange operator support
#	- fix the require() and Net::SMS...-> ack

use strict;

use Getopt::Long;
use Date::Manip;

# Constants
use constant USAGE =><<END_OF_USAGE;
Usage:
	sms4nothing.pl --phone-number=NUM --message="TEXT" [OPTIONS]...
END_OF_USAGE

use constant HELP =><<END_OF_HELP;
Try `sms4nothing.pl --help' for more information.
END_OF_HELP

use constant LONG_HELP =><<END_OF_LONG_HELP;
Arguments:
	-m, --message="TEXT"	your SMS TEXT message (use \\! for !)
	-n, --phone-number=NUM	host phone number

Options:
	-a, --ack=EMAIL		your EMAIL address for acknowledgment
	-d, --date=DATE		deliver message at DATE
	--debug			print debug messages
	-e, --extra-lib-path=DIR	specify DIR as an extra library path
					for modules hierarchy (Net::SMS)
	-f, --file=OPFILE	use ART OPFILE to find appropriate operator
	-h, --help		print this help and exit
	-o, --operator=OPRT	use OPRT module to deliver message
	-p, --proxy=ADDR:PORT	set http proxy ADDR (name or IP) and PORT
				(default is taken from ENV)
	-s, --sender[=NAME]	sender NAME (default is username)
	-v, --verbose		tell what we are doing
	--version		print version number
	-w, --screen-width	screen width when displaying SMS in verbose mode

Indications:
	. message is limited to 160 or 640 depending of the operator
	. phone number must begin with 06 and be ten digit long
	. offset and familiar dates are interpreted
	. mail address syntax is checked when specified
	. default date is now (idem when you make a mistake)
	. default sender is username (use -s alone for anonymous sender)
	. default proxy is taken from ENV (use -p to use another one)
	. default extra library path is the current directory
	. you can specify many extra library pathes using spaces as separators
	. you can also export the environment variable PERL5LIB to set extra
	  library pathes
	. default path to find ART operators file
	  (http://www.art-telecom.fr/interactive/numeros/wopnum.rtf)
	. if you don't like long command lines, use shell aliases ;-)

Examples:
	. sms4nothing.pl -m "My message \\!\\!" -n 0612345678" .
	. sms4nothing.pl -m "Another message" -n 0612345678 -d tomorrow -o SFR
	. sms4nothing.pl -m "Hi" -n 0612345678 -d "21 october 2000 8:00"
	. sms4nothing.pl -m "Hello World" -n 0612345678 -a julien23\@nowhere.com
	  -d 5min -s bill -p http://my_proxy:8081 --debug -v -w 18
	  -e ~/usr/bin/sms -f ~/usr/bin/sms/wopnum.rtf
END_OF_LONG_HELP

use vars qw($opt_recipient $opt_sender $opt_message $opt_ack $opt_date
	$opt_proxy $opt_debug $opt_help $opt_version $opt_verbose $opt_width
	$opt_libpath $opt_operator $opt_file);

my %modules = (
	'SFR' => 'SFR',
	'BOUY' => 'Bouygtel'
);

GetOptions(
	"message=s" => \$opt_message,
	"n=i" => \$opt_recipient,
	"phone-number=i" => \$opt_recipient,
	"ack=s" => \$opt_ack,
	"d=s" => \$opt_date,
	"date=s" => \$opt_date,
	"debug" => \$opt_debug,
	"extra-lib-path=s" => \$opt_libpath,
	"file=s" => \$opt_file,
	"help" => \$opt_help,
	"o=s" => \$opt_operator,
	"operator=s" => \$opt_operator,
	"p=s" => \$opt_proxy,
	"proxy=s" => \$opt_proxy,
	"sender:s" => \$opt_sender,
	"v" => \$opt_verbose,
	"verbose" => \$opt_verbose,
	"version" => \$opt_version,
	"width=i" => \$opt_width
);

print VERSION . "\n" if defined $opt_version;

die USAGE . "\n" . LONG_HELP if defined $opt_help;
die USAGE . "\n" . HELP unless defined $opt_recipient && defined $opt_message;

# Find which operator to use
unless (defined $opt_operator) {
	$opt_file = "./wopnum.rtf" unless defined $opt_file;
	print "ART file: $opt_file\n" if defined $opt_debug;

	open OPERATORS, "<$opt_file" or
		die "Can't open $opt_file to find the appropriate operator.";

	my ($prefix, $operator);
	while (<OPERATORS>) {
		if (($prefix, $operator) = /^"(\d+)"\s*".*?"\s*"(.*?)"/ and
			not index($opt_recipient, $prefix)) {
			$opt_operator = $operator;
			last;
		}
	}

	close OPERATORS;
}

die "Can't find the appropriate operator for $opt_recipient in $opt_file"
	unless defined $opt_operator;

if (defined $modules{$opt_operator}) {
	print "Operator: ", $modules{$opt_operator}, "\n"
		if defined $opt_verbose;
} else {
	die "Unknown or not supported operator $opt_operator";
}

# Path to modules
push @INC, (defined $opt_libpath) ? split(/ /, $opt_libpath) : ".";
require "Net/SMS/" . $modules{$opt_operator} . ".pm";

# Create short message
my $lib = "Net::SMS::" . $modules{$opt_operator};
my $sms = $lib->new();

# Mandatory options
# Sender
$opt_sender =  $ENV{USER} unless defined $opt_sender;
$sms->sender($opt_sender);
print "Sender: $opt_sender\n" if defined $opt_verbose;

# Phone number
$sms->recipient($opt_recipient);
print "Phone number: $opt_recipient\n" if defined $opt_verbose;

# Short message
$sms->message($opt_message);
if (defined $opt_verbose) {
	$opt_width = 14 unless defined $opt_width;
	my @screen = $opt_message =~ /(.{1,$opt_width})/g;
	print "Message:\n",
		join('', map{sprintf "\t|%-${opt_width}s|\n", $_} @screen);
}

# Non mandatory options
# Email for acknowlegment
if (defined $opt_ack) {
	$sms->ack($opt_ack);
	print "Email: $opt_ack\n" if defined $opt_verbose;
}

# Date
if (defined $opt_date) {
	$sms->date($opt_date);
	if (defined $opt_verbose) {
		my $date = UnixDate(ParseDate($opt_date),
			"%A %e %B %Y - %H:%M");
		print "Date: \u$date\n";
	}
}

# Proxy settings
if (defined $opt_proxy) {
	$sms->proxy($opt_proxy);
	print "Proxy: $opt_proxy\n" if defined $opt_verbose;
}

# Debug
$sms->debug($opt_debug) if defined $opt_debug;

# Verbose
$sms->verbose($opt_verbose) if defined $opt_verbose;

# Send the message
$sms->send();
